"use strict";
/**
 * @license
 * Copyright 2019 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tfjs_core_1 = require("@tensorflow/tfjs-core");
const shader_preprocessor_1 = require("../shader_preprocessor");
const webgpu_util_1 = require("../webgpu_util");
exports.MUL = 'return a * b;';
exports.ADD = 'return a + b;';
exports.SUB = 'return a - b;';
exports.DIV = 'return a / b;';
exports.INT_DIV = `
  float s = sign(a) * sign(b);
  int ia = int(round(a));
  int ib = int(round(b));
  return float(idiv(ia, ib, s));
`;
class BinaryOpProgram {
    constructor(op, aShape, bShape) {
        this.variableNames = ['A', 'B'];
        this.workPerThread = 4;
        this.workGroupSize = [16, 1, 1];
        this.outputShape = tfjs_core_1.backend_util.assertAndGetBroadcastShape(aShape, bShape);
        const size = tfjs_core_1.util.sizeFromShape(this.outputShape);
        this.dispatchLayout = webgpu_util_1.flatDispatchLayout(this.outputShape);
        this.dispatch = webgpu_util_1.computeDispatch(this.dispatchLayout, this.outputShape, this.workGroupSize, [this.workPerThread, 1, 1]);
        const type = shader_preprocessor_1.getCoordsDataType(this.outputShape.length);
        this.userCode = `
      float binaryOperation(float a, float b) {
        ${op}
      }

      void main() {
        int index = int(gl_GlobalInvocationID.x);

        for(int i = 0; i < ${this.workPerThread}; i++) {
          int flatIndex = index * ${this.workPerThread} + i;

          if(flatIndex < ${size}) {
            ${type} coords = getCoordsFromFlatIndex(flatIndex);

            float a = getAAtOutCoords(coords);
            float b = getBAtOutCoords(coords);
            setOutput(flatIndex, binaryOperation(a, b));
          }
        }
      }
    `;
    }
}
exports.BinaryOpProgram = BinaryOpProgram;
//# sourceMappingURL=binary_op_webgpu.js.map