"use strict";
/**
 * @license
 * Copyright 2019 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
Object.defineProperty(exports, "__esModule", { value: true });
const shader_preprocessor_1 = require("../shader_preprocessor");
const webgpu_util_1 = require("../webgpu_util");
class TransposeProgram {
    constructor(aShape, newDim) {
        this.variableNames = ['A'];
        const outputShape = new Array(aShape.length);
        for (let i = 0; i < outputShape.length; i++) {
            outputShape[i] = aShape[newDim[i]];
        }
        this.outputShape = outputShape;
        this.rank = outputShape.length;
        const dtype = shader_preprocessor_1.getCoordsDataType(this.rank);
        this.dispatchLayout = webgpu_util_1.flatDispatchLayout(this.outputShape);
        this.dispatch = webgpu_util_1.computeDispatch(this.dispatchLayout, this.outputShape);
        const switched = getSwitchedCoords(newDim);
        this.userCode = `
      void main() {
        ${dtype} resRC = getOutputCoords();
        setOutput(getFlatIndex(resRC, outShape), A[getFlatIndex(
          ${dtype}(${switched}), aShape)]);
      }
    `;
    }
}
exports.TransposeProgram = TransposeProgram;
function getSwitchedCoords(newDim) {
    const rank = newDim.length;
    if (rank > 4) {
        throw Error(`Transpose for rank ${rank} is not yet supported`);
    }
    const switchedCoords = new Array(rank);
    for (let i = 0; i < newDim.length; i++) {
        switchedCoords[newDim[i]] = `resRC[${i}]`;
    }
    return switchedCoords.join();
}
//# sourceMappingURL=transpose_webgpu.js.map